package com.autentia.citas.dao;

import java.io.Serializable;
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

@Stateless
public class JPADaoImpl<T> implements Dao<T>{

	@PersistenceContext(unitName="citas")
	private EntityManager manager;

	public List<T> getAll(Class<T> entityClass) {
		return manager.createQuery("select e from "+entityClass.getName()+" as e",entityClass).getResultList();
	}

	public T findById(Class<T> entityClass, Serializable id) {		
		return manager.find(entityClass, id);
	}

	public List findByQuery(String query, Object...params) {
		Query q = manager.createQuery(query);
		if(params.length>0) {
			int i=1;
			for(Object o:params) {
				q.setParameter(i++, o);
			}
		}
		return q.getResultList();
	}

	public void create(T entity) {
		manager.persist(entity);		
	}

	public void delete(T entity) {
		manager.remove(manager.merge(entity));		
	}

	public T update(T entity) {
		return manager.merge(entity);		
	}
	
}
