package com.autentia.citas.managers;

import java.util.Calendar;
import java.util.List;
import java.util.concurrent.Future;
import javax.annotation.Resource;
import javax.ejb.AsyncResult;
import javax.ejb.Asynchronous;
import javax.ejb.EJB;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.ejb.Timeout;
import javax.ejb.Timer;
import javax.ejb.TimerConfig;
import javax.ejb.TimerService;
import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;
import com.autentia.citas.dao.Dao;
import com.autentia.citas.model.Cita;
import com.autentia.citas.model.Contacto;

@Stateless
public class CitasManager {
	
	@EJB
	private Dao dao;
	
	@Resource
	private SessionContext ctx;
	
	@SuppressWarnings("unchecked")
	public List<Cita> getAllCitas() {
		return dao.getAll(Cita.class);
	}
	
	@SuppressWarnings("unchecked")
	public Cita getCita(Long id) {
		return (Cita) dao.findById(Cita.class, id);
	}
	
	@SuppressWarnings("unchecked")
	public Contacto getContacto(Long id) {
		return (Contacto) dao.findById(Contacto.class, id);
	}
	
	public void citarme(Contacto contacto, Cita cita) {
		if(contacto.getId()!=null) {
			dao.create(cita);
			contacto.addCita(cita);
			dao.update(contacto);
		} else {
			contacto.addCita(cita);
			dao.create(contacto);
		}
		
		crearRecordatorio(cita);		
	}
	
	/**
	 * Mtodo que crea un Timer de recordatorio de la cita una hora antes de la misma
	 * @param cita
	 */
	private void crearRecordatorio(Cita cita) {
		// No uso DI para buscar el TimerService por problemas en el contenedor embebido.
		TimerService timerService = null;
		try {
			timerService = (TimerService) ctx.getTimerService();
		} catch (Exception e) {
			
		}
		
		if(timerService!=null) {
			Calendar cal = Calendar.getInstance();
				cal.setTime(cita.getFecha());
				cal.add(Calendar.HOUR, -1);
				timerService.createSingleActionTimer(cal.getTime(), new TimerConfig(cita, true));
		}
	}
	
	/**
	 * Mtodo de callback del timer.
	 * @param timer
	 */
	@Timeout
	private void sendRecordatorio(Timer timer) {
		Cita c = (Cita) timer.getInfo();
		
		// Ahora enviamos el correo al contacto de la cita y a mi...pero esto no lo voy a hacer
	}
	

	@SuppressWarnings("unchecked")
	public List<Contacto> getAllContactos() {
		return dao.getAll(Contacto.class);
	}
	 
	public List<Cita> getCitasWith(Contacto contacto) {
		return dao.findByQuery("select cita from Cita cita inner join cita.contacto contacto where contacto.id = ?1"
				,contacto.getId());
	}
	
	/**
	 * Mtodo asncrono sin respuesta
	 * @param c
	 */
	@Asynchronous
	public void recuerdaleLaCita(Cita c) {
		// No lo hagis en casa... ni en produccin, es para probar esto...
		try {
			Thread.currentThread().sleep(10000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		// Aqu le mandamos un mail...
		// Lo siento pero paso de hacer esto.
		
	}
	
	/**
	 * Mtodo asncrono con respuesta
	 * @param c
	 */
	@Asynchronous
	public Future<Boolean> recuerdaleLaCitaAck(Cita c) {
		// No lo hagis en casa... ni en produccin, es para probar esto...
		try {
			Thread.currentThread().sleep(10000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		// Aqu le mandamos un mail...
		// Y le decimos que OK	
		// Lo siento pero paso de hacer esto.
		return new AsyncResult<Boolean>(true);
	}
	
	/**
	 * Este mtodo nos contar lo que tardan las cosas.
	 * @param inv
	 * @return
	 */
	@AroundInvoke
	private Object cronometro(InvocationContext inv) {
		long time1 = System.currentTimeMillis();
			try {
				return inv.proceed();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				return null;
			} finally {
				long total = System.currentTimeMillis() - time1;
				System.out.println("La invocacin a:"+inv.getTarget()+"->"+inv.getMethod()+" ha tardado: "+total+" ms");
			}		
	}
	
	
	
}
