package com.autentia.citas.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.OneToMany;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

@Entity
public class Contacto implements Serializable{

	@Id
	@GeneratedValue
	private Long id;
	
	private String nombre;
	
	private String apellidos;	
	
	@ElementCollection(fetch=FetchType.LAZY)	
	private List<String> telefonos = new ArrayList<String>(0);
	
	@ElementCollection(fetch=FetchType.LAZY)
	private List<String> emails =new ArrayList<String>(0);
	
	@OneToMany(mappedBy="contacto",orphanRemoval=true,cascade=CascadeType.PERSIST,fetch=FetchType.EAGER)	
	private List<Cita> citas =new ArrayList<Cita>(0);

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public String getApellidos() {
		return apellidos;
	}

	public void setApellidos(String apellidos) {
		this.apellidos = apellidos;
	}

	public List<String> getTelefonos() {
		return telefonos;
	}
	
	public List<String> getEmails() {
		return emails;
	}	
	
	public List<Cita> getCitas() {
		return citas;
	}	

	public Long getId() {
		return id;
	}
	
	public void addCita(Cita cita) {
		citas.add(cita);
		cita.setContacto(this);
	}
	
	public void addTelefono(String telefono) {
		if(!telefonos.contains(telefono)) {
			telefonos.add(telefono);
		}
	}
	
	public void addEmail(String email) {
		if(!emails.contains(email)) {
			emails.add(email);		
		}
	}
	
	public void removeCita(Cita cita) {
		if(citas.contains(cita)) {
			citas.remove(cita);
			cita.setContacto(null);
		}
	}
	
	public void removeTelefono(String telefono) {
		if(telefonos.contains(telefono)) {
			telefonos.remove(telefono);			
		}
	}
	
	public void removeEmails(String email) {
		if(emails.contains(email)) {
			emails.remove(email);			
		}
	}
	
	@Override
	public int hashCode() {
		return new HashCodeBuilder().append(nombre).append(apellidos).toHashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		final Contacto other = (Contacto)obj;
		return new EqualsBuilder().append(nombre, other.nombre).append(apellidos, other.apellidos).isEquals();
	}
	

}