package com.autentia.citas.dao;

import java.util.Calendar;
import javax.naming.NamingException;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import com.autentia.citas.model.Cita;
import com.autentia.citas.model.Contacto;
import com.autentia.citas.utils.Utils;

public class DaoTest {	
	
	@BeforeClass
	public static void initTests() throws Exception {		
		if(Utils.container==null) {
			Utils.startContainer();
		}
	}
	
	@AfterClass
	public static void releaseTests() {		
		//Utils.closeContainer();
	}

	@Test
	public void createCita() throws NamingException {
		
		Contacto contacto = new Contacto();
			contacto.setNombre("Francisco Javier");
			contacto.setApellidos("Martnez Pez");
			contacto.addEmail("fjmpaez@autentia.com");
			contacto.addEmail("fjmpaez@acme.com");
			contacto.addTelefono("915551111");
			contacto.addTelefono("620999999");		
		
		Cita cita = new Cita();		
			cita.setAsunto("Quedada en el parque con los nios");
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.YEAR, 2014);
			cal.set(Calendar.DAY_OF_MONTH, 21);
			cal.set(Calendar.MONTH, 7);
			cita.setFecha(cal.getTime());		
			
			cal.set(Calendar.YEAR, 2012);
			cal.set(Calendar.DAY_OF_MONTH, 21);
			cal.set(Calendar.MONTH, 7);		
		
		Cita cita2 = new Cita();		
			cita2.setAsunto("Reunin de seguimiento proyecto ACME LA CAMA");
			cita2.setFecha(cal.getTime());
		
		// Usamos el contexto JNDI para buscar el EJB...Y usando el nombre estandarizado...
		Dao dao =(Dao)Utils.ctx.lookup("java:global/citas/JPADaoImpl");
		
		contacto.addCita(cita);
		contacto.addCita(cita2);		
		
		// Persistimos contacto y en cascada se persistirn las citas 
		dao.create(contacto);	
		
		
		Assert.assertTrue(dao.getAll(Cita.class).size()==2);		
		Assert.assertTrue(dao.getAll(Contacto.class).size()==1);
		Assert.assertTrue(dao.findByQuery("select cita from Cita cita inner join cita.contacto contacto where contacto.nombre like 'Francisco Javier'").size()==2);		
	}
	
	
	@Test
	public void deletingAll() throws NamingException {
		Dao dao =(Dao)Utils.ctx.lookup("java:global/citas/JPADaoImpl");
		Contacto contacto = (Contacto) dao.getAll(Contacto.class).get(0);
		
		// El orphans delete entra en juego eliminado las citas
		dao.delete(contacto);
		
		Assert.assertTrue(dao.getAll(Cita.class).size()==0);		
		Assert.assertTrue(dao.getAll(Contacto.class).size()==0);
	}
	
}
